@@ -1,8 +1,11 @@
+import enum
+import logging
 from collections import OrderedDict
+from typing import Union
 
-from .commons import wrap_val
+from .commons import first_in, wrap_val
 from .fields import (
-    StructureReference,
+    FunctionCall, Map, Negative, NonNegative, NonPositive, Positive, StructureReference,
     Integer,
     Number,
     Float,
@@ -21,7 +24,13 @@ from .fields import (
 )
 
 from .extfields import DateString
-from .structures import ADDITIONAL_PROPERTIES
+from .mappers import DoNotSerialize, aggregate_serialization_mappers
+from .structures import ADDITIONAL_PROPERTIES, NoneField
+
+SCHEMA_PATTERN_PROPERTIES = "patternProperties"
+SCHEMA_ADDITIONAL_PROPERTIES = "additionalProperties"
+SCHEMA_PROPETIES = "properties"
+SCHEMA_PROPERTY_NAMES = "propertyNames"
 
 
 def get_mapper(field_cls):
@@ -41,6 +50,7 @@ def get_mapper(field_cls):
         NotField: NotFieldMapper,
         Tuple: ArrayMapper,
         Set: ArrayMapper,
+        Map: MapMapper,
     }
     for cls in field_cls.__mro__:
         if issubclass(cls, Field) and cls in field_type_to_mapper:
@@ -55,7 +65,7 @@ def _map_class_reference(reference, definitions_schema):
     return {"$ref": f"#/definitions/{name}"}
 
 
-def convert_to_schema(field, definitions_schema):
+def convert_to_schema(field, definitions_schema, serialization_mapper: dict = None):
     """
     In case field is None, should return None.
     Should deal with a list of fields, as well as a single one
@@ -65,12 +75,27 @@ def convert_to_schema(field, definitions_schema):
     if isinstance(field, ClassReference):
         return _map_class_reference(field, definitions_schema)
     if isinstance(field, list):
-        return [convert_to_schema(f, definitions_schema) for f in field]
+        return [convert_to_schema(f, definitions_schema, serialization_mapper=serialization_mapper) for f in field]
     mapper = get_mapper(field.__class__)(field)
-    return mapper.to_schema(definitions_schema)
+    return mapper.to_schema(definitions_schema, serialization_mapper=serialization_mapper)
 
 
-def structure_to_schema(structure, definitions_schema):
+def _validated_mapped_value(mapper, key):
+    if key in mapper:
+        key_mapper = mapper[key]
+        if isinstance(key_mapper, (FunctionCall,)):
+            logging.error(f"{key} is mapped to a function  in serialization mapper- "
+                           "This is unsupported by code-to-schema conversion. "
+                           "You will need to manually fix it.")
+        elif key_mapper is DoNotSerialize:
+            return key_mapper
+        elif not isinstance(key_mapper, (FunctionCall, str)):
+            raise TypeError("mapper must have a FunctionCall or a string")
+
+    return None
+
+
+def structure_to_schema(structure, definitions_schema, serialization_mapper = None):
     """
     Generate JSON schema from :class:`Structure`
     `See working examples in test. <https://github.com/loyada/typedpy/tree/master/tests/test_struct_to_schema.py>`_
@@ -90,28 +115,46 @@ def structure_to_schema(structure, definitions_schema):
     """
     # json schema draft4 does not support inheritance, so we don't need to worry about that
     props = structure.__dict__
-    fields = [key for key, val in props.items() if isinstance(val, Field)]
-    required = props.get("_required", fields)
-    additional_props = props.get(ADDITIONAL_PROPERTIES, True)
-    if len(fields) == 1 and required == fields and additional_props is False:
+    field_by_name = structure.get_all_fields_by_name()
+    required = getattr(structure, "_required", list(field_by_name.keys()))
+    additional_props = getattr(structure, ADDITIONAL_PROPERTIES, True)
+    mapper = aggregate_serialization_mappers(structure, serialization_mapper) or {}
+    if len(field_by_name) == 1 and set(required) == set(field_by_name.keys()) and additional_props is False:
+        _, value = first_in(field_by_name.items())
         return (
-            convert_to_schema(props[fields[0]], definitions_schema),
+            convert_to_schema(value, definitions_schema),
             definitions_schema,
         )
     else:
         fields_schema = OrderedDict([("type", "object")])
-        fields_schema["properties"] = OrderedDict(
-            [(key, convert_to_schema(props[key], definitions_schema)) for key in fields]
-        )
+        fields_schema["properties"] = {}
+        properties = fields_schema["properties"]
+        for key, field in field_by_name.items():
+            mapped_key = mapper[key] if key in mapper and isinstance(mapper[key], (str,)) else key
+            mapped_value = _validated_mapped_value(mapper, key)
+            if mapped_value is DoNotSerialize:
+                if mapped_key in required:
+                    required.pop(required.index(mapped_key))
+            if mapped_value is not DoNotSerialize:
+                if key in required:
+                    required[required.index(key)] = mapped_key
+                sub_mapper = mapper.get(f"{key}._mapper", {})
+                sub_schema = convert_to_schema(field, definitions_schema, serialization_mapper=sub_mapper)
+                default_raw = getattr(field,"_default")
+                if default_raw is not None:
+                    default_val = default_raw() if callable(default_raw) else default_raw
+                    sub_schema["default"] = default_val
+                properties[mapped_key] =sub_schema
         fields_schema.update(
             OrderedDict(
                 [
-                    ("required", required),
+                    ("required", sorted(required)),
                     ("additionalProperties", additional_props),
                 ]
             )
         )
-    return (fields_schema, definitions_schema)
+
+    return fields_schema, definitions_schema
 
 
 def convert_to_field_code(schema, definitions):
@@ -129,6 +172,7 @@ def convert_to_field_code(schema, definitions):
         def_name = schema["$ref"][len("#/definitions/") :]
         return def_name
 
+
     type_name_to_field = {
         "object": StructureReference,
         "integer": Integer,
@@ -149,10 +193,21 @@ def convert_to_field_code(schema, definitions):
         cls = Enum
         mapper = get_mapper(cls)
     else:
-        cls = type_name_to_field[schema.get("type", "object")]
+        object_type = schema.get("type", "object")
+        if object_type == "object":
+            if SCHEMA_PROPETIES in schema:
+                cls = StructureReference
+            else:
+                cls = Map
+        else:
+            cls = type_name_to_field[schema.get("type", "object")]
         mapper = get_mapper(cls)
     params_list = mapper.get_paramlist_from_schema(schema, definitions)
-
+    if "default" in schema:
+        default_val = schema["default"]
+        if isinstance(default_val, (list, dict)):
+            default_val = f"lambda: {default_val}"
+        params_list.append(("default", default_val))
     params_as_string = ", ".join([f"{name}={val}" for (name, val) in params_list])
     return f"{cls.__name__}({params_as_string})"
 
@@ -195,7 +250,6 @@ def schema_to_struct_code(struct_name, schema, definitions_schema):
         if schema.get("type", "object") == "object"
         else ["wrapped"]
     )
-    body += [f"    _required = {required}"] if required is not None else []
     the_type = schema.get("type", "object" if "properties" in schema else None)
 
     if the_type == "object":
@@ -205,6 +259,8 @@ def schema_to_struct_code(struct_name, schema, definitions_schema):
     else:
         body += [f"    wrapped = {convert_to_field_code(schema, definitions_schema)}"]
 
+    body += ["", f"    _required = {required}"] if required is not None else []
+
     return "\n".join(body)
 
 
@@ -223,7 +279,7 @@ def schema_definitions_to_code(schema):
     code = []
     for (name, sch) in schema.items():
         code.append(schema_to_struct_code(name, sch, schema))
-    return "\n\n".join(code)
+    return "\n\n\n".join(code)
 
 
 def write_code_from_schema(schema, definitions_schema, filename, class_name):
@@ -251,9 +307,10 @@ def write_code_from_schema(schema, definitions_schema, filename, class_name):
     supporting_classes = schema_definitions_to_code(definitions_schema)
     structure_code = schema_to_struct_code(class_name, schema, definitions_schema)
     with open(filename, "w", encoding="utf-8") as fout:
-        fout.write("from typedpy import *\n\n")
-        fout.write(supporting_classes)
-        fout.write("\n\n# ********************\n\n")
+        fout.write("from typedpy import *\n\n\n")
+        if definitions_schema:
+            fout.write(supporting_classes)
+            fout.write("\n\n# ********************\n\n\n")
         fout.write(structure_code)
         fout.write("\n")
 
@@ -281,8 +338,8 @@ class StructureReferenceMapper(Mapper):
         ]
         return body
 
-    def to_schema(self, definitions):
-        schema, _ = structure_to_schema(getattr(self.value, "_newclass"), definitions)
+    def to_schema(self, definitions, serialization_mapper):
+        schema, _ = structure_to_schema(getattr(self.value, "_newclass"), definitions, serialization_mapper)
         schema["type"] = "object"
         return schema
 
@@ -298,28 +355,96 @@ class NumberMapper(Mapper):
         }
         return list((k, v) for k, v in params.items() if v is not None)
 
-    def to_schema(self, definitions):
+    def to_schema(self, definitions, serialization_mapper):
+        def get_min(value):
+            if value.minimum is not None:
+                return value.minimum
+            if isinstance(value, NonNegative):
+                return 0
+            if isinstance(value, Positive):
+                return 1 if isinstance(value, Integer) else 0.000001
+            return None
+
+        def get_max(value):
+            if value.maximum is not None:
+                return value.maximum
+            if isinstance(value, NonPositive):
+                return 0
+            if isinstance(value, Negative):
+                return -1 if isinstance(value, Integer) else -0.000001
+            return None
+
         value = self.value
         params = {
             "type": "number",
             "multiplesOf": value.multiplesOf,
-            "minimum": value.minimum,
-            "maximum": value.maximum,
+            "minimum": get_min(value),
+            "maximum": get_max(value),
             "exclusiveMaximum": value.exclusiveMaximum,
         }
         return {k: v for k, v in params.items() if v is not None}
 
 
+class MapMapper(Mapper):
+    @staticmethod
+    def get_paramlist_from_schema(schema, definitions):
+        additional_properties = schema.get(SCHEMA_ADDITIONAL_PROPERTIES, None)
+        pattern_properties = schema.get(SCHEMA_PATTERN_PROPERTIES, None)
+        property_names = schema.get(SCHEMA_PATTERN_PROPERTIES, {})
+        if (pattern_properties and len(pattern_properties)>1) or (pattern_properties and additional_properties) or (
+                pattern_properties and property_names):
+            raise NotImplementedError("Conversion for this map is unsupported")
+        if not any([additional_properties, property_names, pattern_properties]):
+            return []
+
+        key_type = convert_to_field_code({**property_names, "type": "string"}, globals()) if property_names else "String()"
+        adjusted_key_type = f"String(pattern='{list(pattern_properties.keys())[0]}')" if pattern_properties else key_type
+        value_type = convert_to_field_code(additional_properties, definitions) if additional_properties else (
+            convert_to_schema(pattern_properties, definitions) if additional_properties else "Anything"
+        )
+
+        items = f"[{adjusted_key_type}, {value_type}]"
+        params = {
+            "items": items,
+            "maxItems": schema.get("maxItems", None),
+            "minItems": schema.get("minItems", None)
+        }
+        return list((k, v) for k, v in params.items() if v is not None)
+
+    def to_schema(self, definitions, serialization_mapper):
+        value = self.value
+        params = {
+            "type": "object",
+        }
+        if value.items:
+            keys, values = value.items
+            if not isinstance(keys, String):
+                raise TypeError("JSON supports only Strings as keys")
+
+            suffix = ''
+            if keys.maxLength or keys.minLength:
+                suffix = f"{{{keys.minLength or ''}, {keys.maxLength or ''}}}"
+            pattern_props = f"{keys.pattern or ''}{suffix}" or None
+            values_schema = convert_to_schema(values, definitions, serialization_mapper=serialization_mapper)
+            if pattern_props:
+                params[SCHEMA_PATTERN_PROPERTIES] = values_schema
+            elif values_schema:
+                params[SCHEMA_ADDITIONAL_PROPERTIES] = values_schema
+        params["maxItems"] = value.maxItems
+        params["minItems"] = value.minItems
+        return {k: v for k, v in params.items() if v is not None}
+
+
 class IntegerMapper(NumberMapper):
-    def to_schema(self, definitions):
-        params = super().to_schema(definitions)
+    def to_schema(self, definitions, serialization_mapper):
+        params = super().to_schema(definitions, serialization_mapper)
         params.update({"type": "integer"})
         return params
 
 
 class FloatMapper(NumberMapper):
-    def to_schema(self, definitions):
-        params = super().to_schema(definitions)
+    def to_schema(self, definitions, serialization_mapper):
+        params = super().to_schema(definitions, serialization_mapper)
         params.update({"type": "float"})
         return params
 
@@ -329,7 +454,7 @@ class BooleanMapper(Mapper):
     def get_paramlist_from_schema(schema, definitions):
         return []
 
-    def to_schema(self, definitions):  # pylint: disable=R0201
+    def to_schema(self, definitions, serialization_mapper):  # pylint: disable=R0201
         params = {
             "type": "boolean",
         }
@@ -346,7 +471,7 @@ class StringMapper(Mapper):
         }
         return list((k, v) for k, v in params.items() if v is not None)
 
-    def to_schema(self, definitions):
+    def to_schema(self, definitions, serialization_mapper):
         value = self.value
         params = {
             "type": "string",
@@ -358,7 +483,7 @@ class StringMapper(Mapper):
 
 
 class DateStringMapper(Mapper):
-    def to_schema(self, definitions):  # pylint: disable=no-self-use
+    def to_schema(self, definitions, serialization_mapper):  # pylint: disable=no-self-use
         params = {
             "type": "string",
             "pattern": r"^([12]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[12]\d|3[01]))$",
@@ -377,14 +502,14 @@ class ArrayMapper(Mapper):
         }
         return list((k, v) for k, v in params.items() if v is not None)
 
-    def to_schema(self, definitions):
+    def to_schema(self, definitions, serialization_mapper):
         value = self.value
         if isinstance(value, Tuple):
             params = {
                 "type": "array",
                 "uniqueItems": value.uniqueItems,
                 "additionalItems": False,
-                "items": convert_to_schema(value.items, definitions),
+                "items": convert_to_schema(value.items, definitions, serialization_mapper),
             }
         elif isinstance(value, Set):
             params = {
@@ -392,7 +517,7 @@ class ArrayMapper(Mapper):
                 "uniqueItems": True,
                 "maxItems": value.maxItems,
                 "minItems": value.minItems,
-                "items": convert_to_schema(value.items, definitions),
+                "items": convert_to_schema(value.items, definitions, serialization_mapper),
             }
         else:
             params = {
@@ -401,7 +526,7 @@ class ArrayMapper(Mapper):
                 "additionalItems": value.additionalItems,
                 "maxItems": value.maxItems,
                 "minItems": value.minItems,
-                "items": convert_to_schema(value.items, definitions),
+                "items": convert_to_schema(value.items, definitions, serialization_mapper),
             }
         return {k: v for k, v in params.items() if v is not None}
 
@@ -414,8 +539,16 @@ class EnumMapper(Mapper):
         }
         return list(params.items())
 
-    def to_schema(self, definitions):
-        params = {"enum": self.value.values}
+    def to_schema(self, definitions, serialization_mapper):
+        def adjust(val) -> Union[str, int, float]:
+            if isinstance(val, enum.Enum):
+                return val.name
+            if not isinstance(val, (int, str, float)):
+                raise TypeError("enum must be an enum, str, or number")
+            return val
+
+        values = [adjust(v) for v in  self.value.values]
+        params = {"enum": values}
         return {k: v for k, v in params.items() if v is not None}
 
 
@@ -428,20 +561,22 @@ class MultiFieldMapper:
 
 
 class AllOfMapper(Mapper):
-    def to_schema(self, definitions):
-        return {"allOf": convert_to_schema(self.value._fields, definitions)}
+    def to_schema(self, definitions, serialization_mapper):
+        return {"allOf": convert_to_schema(self.value._fields, definitions, serialization_mapper)}
 
 
 class OneOfMapper(Mapper):
-    def to_schema(self, definitions):
-        return {"oneOf": convert_to_schema(self.value._fields, definitions)}
+    def to_schema(self, definitions, serialization_mapper):
+        return {"oneOf": convert_to_schema(self.value._fields, definitions, serialization_mapper)}
 
 
 class AnyOfMapper(Mapper):
-    def to_schema(self, definitions):
-        return {"anyOf": convert_to_schema(self.value._fields, definitions)}
+    def to_schema(self, definitions, serialization_mapper):
+        if len(self.value._fields)==2 and self.value._fields[1].__class__ == NoneField:
+            return convert_to_schema(self.value._fields[0], definitions, serialization_mapper)
+        return {"anyOf": convert_to_schema(self.value._fields, definitions, serialization_mapper)}
 
 
 class NotFieldMapper(Mapper):
-    def to_schema(self, definitions):
-        return {"not": convert_to_schema(self.value._fields, definitions)}
+    def to_schema(self, definitions, serialization_mapper):
+        return {"not": convert_to_schema(self.value._fields, definitions, serialization_mapper)}
